@@ -34,6 +34,7 @@ from datetime import datetime
 from queue import Empty, Queue
 from typing import TYPE_CHECKING, Any, Sequence
 
+from kubernetes.dynamic import DynamicClient
 from sqlalchemy import select, update
 
 from airflow.providers.cncf.kubernetes.pod_generator import PodMutationHookException, PodReconciliationError
@@ -74,7 +75,10 @@ except ImportError:
     raise
 from airflow.configuration import conf
 from airflow.executors.base_executor import BaseExecutor
-from airflow.providers.cncf.kubernetes.executors.kubernetes_executor_types import POD_EXECUTOR_DONE_KEY
+from airflow.providers.cncf.kubernetes.executors.kubernetes_executor_types import (
+    ADOPTED,
+    POD_EXECUTOR_DONE_KEY,
+)
 from airflow.providers.cncf.kubernetes.kube_config import KubeConfig
 from airflow.providers.cncf.kubernetes.kubernetes_helper_functions import annotations_to_key
 from airflow.utils.event_scheduler import EventScheduler
@@ -160,19 +164,31 @@ class KubernetesExecutor(BaseExecutor):
         super().__init__(parallelism=self.kube_config.parallelism)
 
     def _list_pods(self, query_kwargs):
+        query_kwargs["header_params"] = {
+            "Accept": "application/json;as=PartialObjectMetadataList;v=v1;g=meta.k8s.io"
+        }
+        dynamic_client = DynamicClient(self.kube_client.api_client)
+        pod_resource = dynamic_client.resources.get(api_version="v1", kind="Pod")
         if self.kube_config.multi_namespace_mode:
             if self.kube_config.multi_namespace_mode_namespace_list:
-                pods = []
-                for namespace in self.kube_config.multi_namespace_mode_namespace_list:
-                    pods.extend(
-                        self.kube_client.list_namespaced_pod(namespace=namespace, **query_kwargs).items
-                    )
+                namespaces = self.kube_config.multi_namespace_mode_namespace_list
             else:
-                pods = self.kube_client.list_pod_for_all_namespaces(**query_kwargs).items
+                namespaces = [None]
         else:
-            pods = self.kube_client.list_namespaced_pod(
-                namespace=self.kube_config.kube_namespace, **query_kwargs
-            ).items
+            namespaces = [self.kube_config.kube_namespace]
+
+        pods = []
+        for namespace in namespaces:
+            # Dynamic Client list pods is throwing TypeError when there are no matching pods to return
+            # This bug was fixed in MR https://github.com/kubernetes-client/python/pull/2155
+            # TODO: Remove the try-except clause once we upgrade the K8 Python client version which
+            # includes the above MR
+            try:
+                pods.extend(
+                    dynamic_client.get(resource=pod_resource, namespace=namespace, **query_kwargs).items
+                )
+            except TypeError:
+                continue
 
         return pods
 
@@ -450,7 +466,7 @@ class KubernetesExecutor(BaseExecutor):
     def _change_state(
         self,
         key: TaskInstanceKey,
-        state: TaskInstanceState | None,
+        state: TaskInstanceState | str | None,
         pod_name: str,
         namespace: str,
         session: Session = NEW_SESSION,
@@ -458,6 +474,15 @@ class KubernetesExecutor(BaseExecutor):
         if TYPE_CHECKING:
             assert self.kube_scheduler
 
+        if state == ADOPTED:
+            # When the task pod is adopted by another executor,
+            # then remove the task from the current executor running queue.
+            try:
+                self.running.remove(key)
+            except KeyError:
+                self.log.debug("TI key not in running: %s", key)
+            return
+
         if state == TaskInstanceState.RUNNING:
             self.event_buffer[key] = state, None
             return
